@@ -68,7 +68,7 @@ int readplanarTIFF(Image * im, TIFF * tif)
 
     for (y = 0; y < im->height; y++)
     {
-        TIFFReadScanline(tif, buf, (uint32) y, 0);
+        TIFFReadScanline(tif, buf, y, 0);
         RGBAtoARGB(buf, im->width, im->bitsPerPixel);
         memcpy(*(im->data) + y * im->bytesPerLine, buf,
                (size_t) (im->bytesPerLine));
@@ -121,7 +121,7 @@ int readtif(Image * im, TIFF * tif)
     }
 
     if (TIFFReadRGBAImage
-        (tif, (uint32) w, (uint32) h, (uint32 *) * (im->data), 0))
+        (tif, w, h, (uint32_t *) * (im->data), 0))
     {
         // Convert agbr to argb; flip image vertically
         unsigned char *cline, *ct, *cb;
@@ -197,8 +197,8 @@ void getCropInformationFromTiff(TIFF * tif, CropInfo * c)
 
     //offset in pixels of "cropped" image from top left corner of 
     //full image (rounded to nearest integer)
-    c->x_offset = (uint32) ((x_position * x_resolution) + 0.49);
-    c->y_offset = (uint32) ((y_position * y_resolution) + 0.49);
+    c->x_offset = (uint32_t) ((x_position * x_resolution) + 0.49);
+    c->y_offset = (uint32_t) ((y_position * y_resolution) + 0.49);
 
     //printf("%s: %dx%d  @ %d,%d", filename, c->cropped_width, c->cropped_height, c->x_offset, c->y_offset);
 }
@@ -385,6 +385,7 @@ int writeCroppedTIFF(Image * im, fullPath * sfile, CropInfo * crop_info)
     case 32:
         TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 8);
         TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, 4);
+        TIFFSetField(tif, TIFFTAG_EXTRASAMPLES, 1, &(uint16_t){EXTRASAMPLE_UNASSALPHA});
         break;
     case 48:
         TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 16);
@@ -393,6 +394,7 @@ int writeCroppedTIFF(Image * im, fullPath * sfile, CropInfo * crop_info)
     case 64:
         TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 16);
         TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, 4);
+        TIFFSetField(tif, TIFFTAG_EXTRASAMPLES, 1, &(uint16_t){EXTRASAMPLE_UNASSALPHA});
         break;
     case 96:
         TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 32);
@@ -402,6 +404,7 @@ int writeCroppedTIFF(Image * im, fullPath * sfile, CropInfo * crop_info)
     case 128:
         TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 32);
         TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, 4);
+        TIFFSetField(tif, TIFFTAG_EXTRASAMPLES, 1, &(uint16_t){EXTRASAMPLE_UNASSALPHA});
         TIFFSetField(tif, TIFFTAG_SAMPLEFORMAT, SAMPLEFORMAT_IEEEFP);
         break;
     }
@@ -617,7 +620,7 @@ int panoTiffGetCropInformation(pano_Tiff * file)
 	    return FALSE;
 	}
 	if (TIFFGetField(file->tiff, TIFFTAG_YRESOLUTION, &y_resolution) == 0) {
-	    PrintError("Cropped image contains XPosition and YPosition, but it does not contain Y Resultion. "
+	    PrintError("Cropped image contains XPosition and YPosition, but it does not contain Y Resolution. "
 		       "Report to developers you think this is a bug");
 	    return FALSE;
 	}
@@ -632,8 +635,8 @@ int panoTiffGetCropInformation(pano_Tiff * file)
 	//full image (rounded to nearest integer)
 	// The position of the file is given in "real" dimensions, we have to rescale them
 
-	c->xOffset = (uint32) ((x_position * x_resolution) + 0.49);
-	c->yOffset = (uint32) ((y_position * y_resolution) + 0.49);
+	c->xOffset = (uint32_t) ((x_position * x_resolution) + 0.49);
+	c->yOffset = (uint32_t) ((y_position * y_resolution) + 0.49);
 
 	// One problem is that some images do not contain FULL size. if they don't have
 	// then assume it
@@ -1077,6 +1080,9 @@ int panoTiffSetImageProperties(pano_Tiff * file)
 			metadata->imageTotalNumber);
 
 
+    if (returnValue && metadata->samplesPerPixel == 4) {
+        returnValue = TIFFSetField(tiffFile, TIFFTAG_EXTRASAMPLES, 1, &(uint16_t){EXTRASAMPLE_UNASSALPHA});
+    }
 
 
     if (returnValue && metadata->bitsPerSample == 32)
@@ -1096,7 +1102,7 @@ int panoTiffSetImageProperties(pano_Tiff * file)
     if (returnValue && metadata->iccProfile.size > 0) {
         returnValue =
             TIFFSetField(tiffFile, TIFFTAG_ICCPROFILE,
-             (uint32)metadata->iccProfile.size,
+             metadata->iccProfile.size,
              (void*)(metadata->iccProfile.data));
              //100, data);
     }
@@ -1141,7 +1147,7 @@ int panoTiffSetImageProperties(pano_Tiff * file)
 int panoTiffReadPlannar(Image * im, pano_Tiff * tif)
 {
     uint8_t *buf;
-    uint32 row;
+    uint32_t row;
     short samplesPerPixel;
     int bytesRead;
     int bitsPerPixel;
@@ -1325,9 +1331,9 @@ int panoTiffReadData(Image * im, pano_Tiff * tif)
     // I changed the stopOnError to 1 so the function reports an error
     // as soon as it happens
 
-    if (TIFFReadRGBAImage(tif->tiff, (uint32) panoTiffImageWidth(tif), 
-                          (uint32) panoTiffImageHeight(tif), 
-                          (uint32 *) * (im->data), 1)) {
+    if (TIFFReadRGBAImage(tif->tiff, (uint32_t) panoTiffImageWidth(tif), 
+                          (uint32_t) panoTiffImageHeight(tif), 
+                          (uint32_t *) * (im->data), 1)) {
         // Convert agbr to argb; flip image vertically
 
         unsigned char *cline, *ct, *cb;
